@@ -12,6 +12,7 @@ from mlagents_envs.side_channel.stats_side_channel import EnvironmentStats
 from mlagents.trainers.policy import Policy
 from mlagents.trainers.agent_processor import AgentManager, AgentManagerQueue
 from mlagents.trainers.action_info import ActionInfo
+from mlagents.trainers.settings import TrainerSettings
 from mlagents_envs.logging_util import get_logger
 
 AllStepResult = Dict[BehaviorName, Tuple[DecisionSteps, TerminalSteps]]
@@ -76,6 +77,17 @@ class EnvManager(ABC):
         """
         pass
 
+    def on_training_started(
+        self, behavior_name: str, trainer_settings: TrainerSettings
+    ) -> None:
+        """
+        Handle traing starting for a new behavior type. Generally nothing is necessary here.
+        :param behavior_name:
+        :param trainer_settings:
+        :return:
+        """
+        pass
+
     @property
     @abstractmethod
     def training_behaviors(self) -> Dict[BehaviorName, BehaviorSpec]:
@@ -107,8 +119,7 @@ class EnvManager(ABC):
                     _policy = self.agent_managers[brain_name].policy_queue.get_nowait()
             except AgentManagerQueue.Empty:
                 if _policy is not None:
-                    # policy_queue contains Policy, but we need a TFPolicy here
-                    self.set_policy(brain_name, _policy)  # type: ignore
+                    self.set_policy(brain_name, _policy)
         # Step the environments
         new_step_infos = self._step()
         return new_step_infos
