@@ -8,6 +8,8 @@ from mlagents.trainers.settings import BehavioralCloningSettings, ScheduleType
 from mlagents.trainers.torch.agent_action import AgentAction
 from mlagents.trainers.torch.action_log_probs import ActionLogProbs
 from mlagents.trainers.torch.utils import ModelUtils
+from mlagents.trainers.trajectory import ObsUtil
+from mlagents.trainers.buffer import AgentBuffer
 
 
 class BCModule:
@@ -35,12 +37,11 @@ class BCModule:
         self.decay_learning_rate = ModelUtils.DecayedValue(
             learning_rate_schedule, self.current_lr, 1e-10, self._anneal_steps
         )
-        params = self.policy.actor_critic.parameters()
+        params = self.policy.actor.parameters()
         self.optimizer = torch.optim.Adam(params, lr=self.current_lr)
         _, self.demonstration_buffer = demo_to_buffer(
             settings.demo_path, policy.sequence_length, policy.behavior_spec
         )
-
         self.batch_size = (
             settings.batch_size if settings.batch_size else default_batch_size
         )
@@ -137,14 +138,18 @@ class BCModule:
         return bc_loss
 
     def _update_batch(
-        self, mini_batch_demo: Dict[str, np.ndarray], n_sequences: int
+        self, mini_batch_demo: AgentBuffer, n_sequences: int
     ) -> Dict[str, float]:
         """
         Helper function for update_batch.
         """
-        vec_obs = [ModelUtils.list_to_tensor(mini_batch_demo["vector_obs"])]
+        np_obs = ObsUtil.from_buffer(
+            mini_batch_demo, len(self.policy.behavior_spec.observation_specs)
+        )
+        # Convert to tensors
+        tensor_obs = [ModelUtils.list_to_tensor(obs) for obs in np_obs]
         act_masks = None
-        expert_actions = AgentAction.from_dict(mini_batch_demo)
+        expert_actions = AgentAction.from_buffer(mini_batch_demo)
         if self.policy.behavior_spec.action_spec.discrete_size > 0:
 
             act_masks = ModelUtils.list_to_tensor(
@@ -161,21 +166,8 @@ class BCModule:
         if self.policy.use_recurrent:
             memories = torch.zeros(1, self.n_sequences, self.policy.m_size)
 
-        if self.policy.use_vis_obs:
-            vis_obs = []
-            for idx, _ in enumerate(
-                self.policy.actor_critic.network_body.visual_processors
-            ):
-                vis_ob = ModelUtils.list_to_tensor(
-                    mini_batch_demo["visual_obs%d" % idx]
-                )
-                vis_obs.append(vis_ob)
-        else:
-            vis_obs = []
-
         selected_actions, log_probs, _, _ = self.policy.sample_actions(
-            vec_obs,
-            vis_obs,
+            tensor_obs,
             masks=act_masks,
             memories=memories,
             seq_len=self.policy.sequence_length,
